@@ -29,13 +29,6 @@ class ForceInline {
   const ForceInline();
 }
 
-// Ensures that the annotated method is represented internally using
-// IR nodes ([:value == true:]) or AST nodes ([:value == false:]).
-class IrRepresentation {
-  final bool value;
-  const IrRepresentation(this.value);
-}
-
 /// Marks a class as native and defines its JavaScript name(s).
 class Native {
   final String name;
@@ -43,19 +36,8 @@ class Native {
 }
 
 class _Patch {
-  final String version;
-
-  const _Patch(this.version);
+  const _Patch();
 }
 
 /// Annotation that marks the declaration as a patch.
-const _Patch patch = const _Patch(null);
-
-/// Annotation that marks the declaration as a patch for the full emitter.
-const _Patch patch_full = const _Patch('full');
-
-/// Annotation that marks the declaration as a patch for the lazy emitter.
-const _Patch patch_lazy = const _Patch('lazy');
-
-/// Annotation that marks the declaration as a patch for the startup emitter.
-const _Patch patch_startup = const _Patch('startup');
+const _Patch patch = const _Patch();
