@@ -6,11 +6,15 @@ part of dart.collection;
 
 /** Common parts of [HashSet] and [LinkedHashSet] implementations. */
 abstract class _HashSetBase<E> extends SetBase<E> {
-
   // The following two methods override the ones in SetBase.
   // It's possible to be more efficient if we have a way to create an empty
   // set of the correct type.
 
+  Set<E> _newSet();
+
+  Set<R> _newSimilarSet<R>();
+
+  Set<R> cast<R>() => Set.castFrom<E, R>(this, newSet: _newSimilarSet);
   Set<E> difference(Set<Object> other) {
     Set<E> result = _newSet();
     for (var element in this) {
@@ -27,8 +31,6 @@ abstract class _HashSetBase<E> extends SetBase<E> {
     return result;
   }
 
-  Set<E> _newSet();
-
   // Subclasses can optimize this further.
   Set<E> toSet() => _newSet()..addAll(this);
 }
@@ -96,9 +98,10 @@ abstract class HashSet<E> implements Set<E> {
    * and the `isValidKey` defaults to accepting all keys.
    * Such a map can be created directly using [HashSet.identity].
    */
-  external factory HashSet({bool equals(E e1, E e2),
-                            int hashCode(E e),
-                            bool isValidKey(potentialKey)});
+  external factory HashSet(
+      {bool equals(E e1, E e2),
+      int hashCode(E e),
+      bool isValidKey(potentialKey)});
 
   /**
    * Creates an unordered identity-based set.
@@ -118,23 +121,34 @@ abstract class HashSet<E> implements Set<E> {
    * two entries that are equal, but not identical, then the first one is
    * the one in the resulting set.
    *
-   * All the [elements] should be assignable to [E].
+   * All the [elements] should be instances of [E].
    * The `elements` iterable itself may have any element type, so this
    * constructor can be used to down-cast a `Set`, for example as:
-   *
-   *     Set<SuperType> superSet = ...;
-   *     Set<SubType> subSet =
-   *         new HashSet<SubType>.from(superSet.where((e) => e is SubType));
+   * ```dart
+   * Set<SuperType> superSet = ...;
+   * Set<SubType> subSet =
+   *     new HashSet<SubType>.from(superSet.whereType<SubType>());
+   * ```
    */
   factory HashSet.from(Iterable elements) {
     HashSet<E> result = new HashSet<E>();
     for (final e in elements) {
-      E element = e as Object/*=E*/;
-      result.add(element);
+      result.add(e);
     }
     return result;
   }
 
+  /**
+   * Create a hash set containing all [elements].
+   *
+   * Creates a hash set as by `new HashSet<E>()` and adds all given [elements]
+   * to the set. The elements are added in order. If [elements] contains
+   * two entries that are equal, but not identical, then the first one is
+   * the one in the resulting set.
+   */
+  factory HashSet.of(Iterable<E> elements) =>
+      new HashSet<E>()..addAll(elements);
+
   /**
    * Provides an iterator that iterates over the elements of this set.
    *
