@@ -74,6 +74,7 @@ class FormatException implements Exception {
    * Optionally also supply the actual [source] with the incorrect format,
    * and the [offset] in the format where a problem was detected.
    */
+  @pragma("vm:entry-point")
   const FormatException([this.message = "", this.source, this.offset]);
 
   /**
@@ -116,19 +117,19 @@ class FormatException implements Exception {
     }
     int lineNum = 1;
     int lineStart = 0;
-    bool lastWasCR;
+    bool previousCharWasCR = false;
     for (int i = 0; i < offset; i++) {
       int char = source.codeUnitAt(i);
       if (char == 0x0a) {
-        if (lineStart != i || !lastWasCR) {
+        if (lineStart != i || !previousCharWasCR) {
           lineNum++;
         }
         lineStart = i + 1;
-        lastWasCR = false;
+        previousCharWasCR = false;
       } else if (char == 0x0d) {
         lineNum++;
         lineStart = i + 1;
-        lastWasCR = true;
+        previousCharWasCR = true;
       }
     }
     if (lineNum > 1) {
@@ -174,6 +175,7 @@ class FormatException implements Exception {
 
 // Exception thrown when doing integer division with a zero divisor.
 class IntegerDivisionByZeroException implements Exception {
+  @pragma("vm:entry-point")
   const IntegerDivisionByZeroException();
   String toString() => "IntegerDivisionByZeroException";
 }
