@@ -11,11 +11,14 @@ part of dart.core;
  * It is a compile-time error for a class to attempt to extend or implement
  * Null.
  */
+@pragma("vm:entry-point")
 class Null {
   factory Null._uninstantiable() {
     throw new UnsupportedError('class Null cannot be instantiated');
   }
 
+  external int get hashCode;
+
   /** Returns the string `"null"`. */
   String toString() => "null";
 }
